<?php

require_once ('class-mobip-api.php');

class Mobip_Payment_Gateway extends WC_Payment_Gateway {

	public function __construct() {
		$this->id = 'mobip_payment_gateway';
		$this->method_title = __('Mobip', 'mobip');
		$this->method_description = __('Take payments with minimum fees on your store using Mobip.', 'mobip');
		
		$this->init_form_fields();
		$this->init_settings();

		$this->title = $this->get_option('title');
		$this->description = $this->get_option('description');
		$this->api_account_id = $this->get_option('api_account_id');
		$this->api_account_secret = $this->get_option('api_account_secret');
		$this->test_mode = $this->get_option('test_mode');
		if ('yes' == $this->test_mode) {
			$this->url = MOBIP_API_SANDBOX_URL;
		} else {
			$this->url = MOBIP_API_PROD_URL;
		}
		$this->mobip_api = new MobipAPI($this->api_account_id, $this->api_account_secret, $this->url);
		
		if ($this->mobip_api) {
			$this->api_account_connected = !empty($this->mobip_api->mobip_api_get_token());
		}

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_order_details_after_order_table', array($this, 'mobip_order_meta_data'));

	}

	public function init_form_fields() {
		$this->form_fields = array(
			'enabled' => array(
				'title' => __('Enable/Disable', 'mobip'),
				'type' => 'checkbox',
				'label' => __('Enable Mobip', 'mobip'),
				'default' => 'yes',
			),
			'title' => array(
				'title' => __('Title', 'mobip'),
				'type' => 'text',
				'description' => __('This controls how customers see as a title for this payment method.', 'mobip'),
				'default' => __('Mobip', 'mobip'),
				'desc_tip' => true,
			),
			'description' => array(
				'title' => __('Description', 'mobip'),
				'type' => 'textarea',
				'description' => __('This controls how customers see as a description for this payment method.', 'mobip'),
				'default' => __('Pay securely via Mobip with your credit card.', 'mobip'),
				'desc_tip' => true,
			),
			'test_mode' => array(
				'title' => __('Test mode', 'mobip'),
				'type' => 'checkbox',
				'label' => __('Enable test mode', 'mobip'),
				/* translators: %s: the sandbox link */
				'description' => sprintf(__('Check this box to use the test account, you can create one <a href="%s" target="_blank">here</a>.', 'mobip'), 'https://sandbox.mobip.fr/'),
			),
			'api_account_id' => array(
				'title' => __('API account id', 'mobip'),
				'type' => 'text',
				'description' => __('Your Mobip account id, can be found in the settings of your account in Mobip website.', 'mobip'),
				'default' => '',
				'desc_tip' => true,
			),
			'api_account_secret' => array(
				'title' => __('API account key', 'mobip'),
				'type' => 'text',
				'description' => __('Your Mobip account key, can be found in the settings of your account in Mobip website.', 'mobip'),
				'default' => '',
				'desc_tip' => true,
			)
		);
	}

	public function admin_options() {
		parent::admin_options();
		?>
		<table class="form-table">
			<tr valign="top">
				<th scope="row" class="titledesc"></th>
				<td class="forminp">
					<button type="button" class="button" id="mobip-test-connection"><?php esc_html_e( 'Test connection', 'mobip' ); ?></button>
					<span id="mobip-test-connection-result" style="line-height: 30px; margin-left: 10px;"></span>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" class="titledesc"><?php esc_html_e( 'API account status', 'mobip' ); ?></th>
				<td class="forminp">
					<?php 
					if (isset($this->api_account_connected) && $this->api_account_connected) { 
						'<span style="color:green">' . esc_html_e( 'Connected', 'mobip' ) . '</span>'; 
					} else { 
						esc_html_e( 'Not connected', 'mobip' ); 
					} 
					?>
				</td>
			</tr>
		</table>
	
		<script>
			jQuery(document).ready(function($) {
				$('#mobip-test-connection').click(function() {

					var account_id = jQuery('#woocommerce_mobip_payment_gateway_api_account_id').val();
					var account_secret = jQuery('#woocommerce_mobip_payment_gateway_api_account_secret').val();
					var test_mode = jQuery('#woocommerce_mobip_payment_gateway_test_mode').is(':checked');
					if (account_id == '' || account_secret == '') $('#mobip-test-connection-result').css("color", "red").html('<?php esc_html_e( 'Account id and key are required.', 'mobip' ); ?>');
					else {
		
						account_id = account_id.toLowerCase();
						account_secret = account_secret.toLowerCase();

						$.ajax({
							url: '<?php echo esc_html_e(get_site_url()); ?>/wp-json/mobip/v1/test-connection',
							method: 'GET',
							data: {
								'account_id': account_id,
								'account_secret': account_secret,
								'test_mode': test_mode
							},
							beforeSend: function() {
								$('#mobip-test-connection-result').css("color", "black").html('<?php esc_html_e( 'Testing the connection ...', 'mobip' ); ?>');
							},
							success: function(response) {
								if (response) {
									$('#mobip-test-connection-result').css("color", "green").html('<?php esc_html_e( 'Connection succeeded.', 'mobip' ); ?>');
								}
								else {
									$('#mobip-test-connection-result').css("color", "red").html('<?php esc_html_e( 'Connection failed.', 'mobip' ); ?>');
								}
							},
							error: function(jqXHR, textStatus, errorThrown) {
								$('#mobip-test-connection-result').css("color", "red").html('<?php esc_html_e( 'Connection failed.', 'mobip' ); ?>');
							}
						});

					}
				});
			});
		</script>
		<?php
	}
	
	public function payment_fields() {
		if ($this->description) {
			echo esc_html_e(wptexturize($this->description));
		}
	}

	public function process_payment( $order_id ) {
		$order = wc_get_order($order_id);

		$items_array = [];
		$items = $order->get_items();
		foreach ( $items as $item ) {
			$items_array[] = $item->get_name() . ' x ' . $item->get_quantity();
		}
		/* translators: %s: the order id */
		$item_name = sprintf(__('Order ID: %s | Items: ', 'mobip'), $order_id) . implode(', ', $items_array);
		if (strlen($item_name) > 500) {
			$item_name = substr($item_name, 0, 497) . '...';
		}

		$customer_name = '';
		$customer_email = '';
		$customer_phone = '';
		
		if (!empty($order->get_billing_first_name()) && !empty($order->get_billing_last_name())) {
			$customer_name = $order->get_billing_first_name() . ' ' . $order->get_billing_last_name();
		}
		if (!empty($order->get_billing_email())) {
			$customer_email = $order->get_billing_email();
		}
		if (!empty($order->get_billing_phone())) {
			$customer_phone = $order->get_billing_phone();
		}

		$order_url = wc_get_account_endpoint_url('view-order') . $order_id;
		$payment = $this->mobip_api->mobip_api_send_request($order_id, $item_name, $order->get_total()*100, $order_url, $customer_name, $customer_email, $customer_phone);
		
		if ($payment) {
			$checkout_url = $payment['url_checkout'];
			$qr_code_url = $payment['url_qr'];
			$transaction_id = $payment['trid'];
			$order->update_meta_data( 'mobip_checkout_url', $checkout_url );
			$order->update_meta_data( 'mobip_qr_code_url', $qr_code_url );
			$order->update_meta_data( 'mobip_transaction_id', $transaction_id );
			/* translators: %s: the transaction id */
			$order->add_order_note(sprintf(__('Mobip transaction created (transaction ID: %s)', 'mobip' ), $transaction_id));
			$order->save();
			$order->update_status('pending', __('Mobip payment is pending.', 'mobip'));
		} else {
			$order->update_status('pending', __('Payment is pending.', 'mobip'));
		}

		if ( !empty($checkout_url) ) {
			$redirect_url = $checkout_url;
		} else {
			$redirect_url = $this->get_return_url($order);
		}

		wc_reduce_stock_levels($order_id);
		WC()->cart->empty_cart();

		return array(
			'result' => 'success',
			'redirect' => $redirect_url,
		);
	}

	public function mobip_order_meta_data( $order ) {
		$checkout_url = $order->get_meta( 'mobip_checkout_url' );
		$qr_code_url = $order->get_meta( 'mobip_qr_code_url' );

		if ($checkout_url || $qr_code_url) {
			echo '<section>';
			echo '<h2 class="woocommerce-order-details__title">' . esc_html_e( 'Mobip payment link', 'mobip' ) . '</h2>';
			if ($checkout_url) {
				echo '<p><a target="_blank" href="' . esc_html_e($checkout_url) . '">' . esc_html_e( 'Click here to open the payment link.', 'mobip' ) . '</a></p>';
			}
			if ($qr_code_url) {
				try {
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $qr_code_url);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					curl_setopt($ch, CURLOPT_HEADER, false);
					$body = curl_exec($ch);
					curl_close($ch);
					$dom = new DOMDocument();
					$dom->loadHTML($body);
					$text = '';
					foreach ($dom->getElementsByTagName('body')->item(0)->childNodes as $node) {
						if (XML_TEXT_NODE === $node->nodeType) {
							$text .= $node->textContent;
						}
					}
					$text = trim($text);
					if (!empty($text)) {
						echo '<p><img style="max-height: 200px;" src="data:image/jpeg;base64,' . esc_html_e($text) . '" alt="QR Code"></p>';
					}
				} catch (\Exception $e) {
					error_log('Exception: ' . $e->getMessage());
				}
			}
			echo '</section>';
		}
		
		
	}
}
