<?php
/**
 * Plugin Name: Mobip
 * Description: Take payments with minimum fees on your store using Mobip.
 * Version: 1.0.0
 * Author: Mobip
 * Author URI: https://mobip.fr/
 * Developer: Mobip
 * Developer URI: https://mobip.fr/
 * Text Domain: mobip
 * Domain Path: /languages
 */

if ( ! defined( 'ABSPATH' ) ) {
  exit; // Exit if accessed directly.
}

define('MOBIP_API_PROD_URL', 'https://app.mobip.fr');
define('MOBIP_API_SANDBOX_URL', 'https://sandbox.mobip.fr');
define('MOBIP_API_TOKEN', '/api/2.0/mobip/get_token');
define('MOBIP_API_ONE_USE_QR', '/api/2.0/mobip/create_one_use_qr');
define('MOBIP_API_IS_VALID', '/api/2.0/mobip/is_valid');

add_action( 'plugins_loaded', 'mobip_payment_gateway_init' );
function mobip_payment_gateway_init() {

	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	require_once 'includes/class-mobip-payment-gateway.php';
	require_once 'includes/class-mobip-api.php';

	add_filter( 'woocommerce_payment_gateways', 'add_mobip_payment_gateway' );
	function add_mobip_payment_gateway( $gateways ) {
		$gateways[] = 'Mobip_Payment_Gateway';
		return $gateways;
	}

	add_filter( 'woocommerce_payment_gateways', 'remove_mobip_payment_gateway' );
	function remove_mobip_payment_gateway( $gateways ) {
		unset( $gateways['Mobip_Payment_Gateway'] );
		return $gateways;
	}

	load_plugin_textdomain( 'mobip', false, plugin_basename( dirname( __FILE__ ) ) . '/languages');

}

register_activation_hook( __FILE__, 'activate_mobip_payment_gateway' );
function activate_mobip_payment_gateway() {
	add_filter( 'woocommerce_payment_gateways', 'add_mobip_payment_gateway' );
}

register_deactivation_hook( __FILE__, 'deactivate_mobip_payment_gateway' );
function deactivate_mobip_payment_gateway() {
	remove_filter( 'woocommerce_payment_gateways', 'add_mobip_payment_gateway' );
}

add_action( 'rest_api_init', 'mobip_register_routes' );
function mobip_register_routes() {
	register_rest_route( 'mobip/v1', '/test-connection', array(
		'methods' => 'GET',
		'callback' => 'mobip_test_connection_callback',
	) );
	register_rest_route( 'mobip/v1', '/payment-callback', array(
		'methods' => 'GET',
		'callback' => 'mobip_payment_callback',
	) );
}

function mobip_test_connection_callback( $request ) {
	try {
		$params = $request->get_params();
		$account_id = $params['account_id'];
		$account_secret = $params['account_secret'];
		if ('true' == $params['test_mode']) {
			$mobip_api = new MobipAPI($account_id, $account_secret, MOBIP_API_SANDBOX_URL);
		} else {
			$mobip_api = new MobipAPI($account_id, $account_secret, MOBIP_API_PROD_URL);
		}
		if ($mobip_api->mobip_api_get_token()) {
			return true;
		}
	} catch (\Exception $e) {
		error_log('Exception: ' . $e->getMessage());
	}
	return false;
}

function mobip_payment_callback( $request ) {
	try {
		if (!empty($request->get_param('trid'))) {
			$trid = $request->get_param('trid');
			$payment_gateways = WC_Payment_Gateways::instance();
			$gateways = $payment_gateways->payment_gateways();
			if (isset($gateways['mobip_payment_gateway'])) {
				$gateway = $gateways['mobip_payment_gateway'];
				if (isset($gateway->api_account_id) && isset($gateway->api_account_secret)) {
					$mobip_api = new MobipAPI($gateway->api_account_id, $gateway->api_account_secret, $gateway->url);
					$is_valid = $mobip_api->mobip_api_is_valid($trid);
					if (isset($is_valid['transaction_status']) && 'PAID' == $is_valid['transaction_status']) {
						$args = array(
							'meta_key' => 'mobip_transaction_id',
							'meta_value' => $trid,
							'post_type' => 'shop_order',
							'post_status' => array_keys( wc_get_order_statuses() ),
							'numberposts' => 1
						);
						$orders = wc_get_orders($args);
						if (!empty($orders)) {
							$order = $orders[0];
							$order->update_status('processing', __('Mobip payment is processed.', 'mobip'));
							return rest_ensure_response(['message' => 'Order updated successfully.']);
						}
					}
					if (isset($is_valid['return_message'])) {
						return rest_ensure_response(['message' => $is_valid['return_message']]);
					}
				}
			}
		}
	} catch (\Exception $e) {
		error_log('Exception: ' . $e->getMessage());
	}
	return rest_ensure_response(['message' => 'Order not updated.']);
}
